# -*- coding: utf-8 -*-

from __future__ import unicode_literals

'''
zqy's utility functions
'''
__version__ = "0.1.7"
__short_description__ = "zqy's utility functions."
__license__ = "MIT"
__author__ = "qianyizhang"
__author_email__ = "landy123007@hotmail.com"
__maintainer__ = "qianyizhang"
__maintainer_email__ = "landy123007@hotmail.com"
__github_username__ = "qianyizhang"
__github_url__ = "https://github.com/qianyizhang/zqy-utils"
__support__ = "https://github.com/qianyizhang/zqy-utils/issues"

from .debug import *    # noqa
from .dicom import *    # noqa
from .dict import *     # noqa
from .io import *       # noqa
from .plot import *     # noqa
from .point import *    # noqa
from .registry import * # noqa


if __name__ == '__main__':
    pass
